/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:		NULL_FileManager
//
// COMMENTS:
//		 A NULL file manager which dumps stuff on the floor
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <CPI.h>

static CPI_Bool			 NullGetFileIDList( CPI_PrivateData	 myHandle,
											CPI_FileIDList	 dirIDList,
											CPI_FileIDList	 fileIDList,
											const char		*dirID,
											CPI_Uint32		 flags,
											CPI_ExtListFn	 fileExtFn );

static CPI_Metadata		 NullGetMetadataToOpen( CPI_PrivateData	 myHandle,
												const char		*fileID,
												CPI_Float32		 curSample );

static const char		*NullGetDeviceName( void );

static CPI_Bool			 NullCreateDirID( CPI_PrivateData	 myHandle,
										  const char		*newdirID,
										  CPI_Bool			 createParent );

static CPI_Bool			 NullRemoveDirID( CPI_PrivateData	 myHandle,
										  const char		*dirID );

static CPI_Bool			 NullRemoveFileID( CPI_PrivateData	 myHandle,
										   const char		*fileID );

static CPI_Bool			 NullRenameFileID( CPI_PrivateData	 myHandle,
										   const char		*oldID,
										   const char		*newID );
static char				*NullTemporaryFileID( CPI_PrivateData	 myHandle,
											  const char		*fileID );

typedef struct _deviceState
{
	CPI_Uint64		theFileLen;
	CPI_Uint64		theCurPos;
	CPI_Bool		theIsOpen;
} deviceState;


static CPI_PrivateData	 NullOpen( CPI_Metadata data );
static void				 NullClose( CPI_PrivateData handle );
static CPI_Bool			 NullIsOpen( CPI_PrivateData handle );
static void				 NullSync( CPI_PrivateData handle );
static CPI_Uint64		 NullGetPos( CPI_PrivateData handle );
static CPI_Uint64		 NullSetPos( CPI_PrivateData	 handle,
									 CPI_Int64			 pos,
									 CPI_Uint8			 whence );
static CPI_Bool			 NullGetFileLength( CPI_PrivateData	 handle,
											CPI_Uint64		*length );
static CPI_Uint64		 NullRead( CPI_PrivateData	 handle,
								   void				*buffer,
								   CPI_Uint64		 bytecount );
static CPI_Uint64		 NullWrite( CPI_PrivateData	 handle,
									const void			*buffer,
									CPI_Uint64			 bytecount );

////////////////////////////////////////


static CPI_Bool
NullGetFileIDList( CPI_PrivateData	 /*myHandle*/,
				   CPI_FileIDList	 /*dirIDList*/,
				   CPI_FileIDList	 fileIDList,
				   const char		* /*dirID*/,
				   CPI_Uint32		 /*flags*/,
				   CPI_ExtListFn	 /*fileExtFn*/ )
{
	cpiAddFileIDToList( fileIDList, "NULL" );

	return CPI_TRUE;
}


////////////////////////////////////////


static CPI_Metadata
NullGetMetadataToOpen( CPI_PrivateData	 myHandle,
					   const char		*fileID,
					   CPI_Float32		 curSample )
{
	CPI_Metadata	 retval			= NULL;

	retval = cpiCreateMetadata();
	if ( NULL != retval )
	{
		cpiSetFilename( retval, "NULL" );
		cpiSetDeviceName( retval, NullGetDeviceName() );
	}

	return retval;
}


////////////////////////////////////////


static const char *
NullGetDeviceName( void )
{
	return "nulldevice";
}


///////////////////////////////////////


static CPI_Bool
NullCreateDirID( CPI_PrivateData	 /*myHandle*/,
				 const char			* /*newdirID*/,
				 CPI_Bool			 /*createParent*/ )
{
	return CPI_TRUE;
}


////////////////////////////////////////


static CPI_Bool
NullRemoveDirID( CPI_PrivateData /*myHandle*/, const char * /*dirID*/ )
{
	return CPI_TRUE;
}


///////////////////////////////////////


static CPI_Bool
NullRemoveFileID( CPI_PrivateData /*myHandle*/, const char * /*fileID*/ )
{
	return CPI_TRUE;
}


////////////////////////////////////////


static CPI_Bool
NullRenameFileID( CPI_PrivateData	 /*myHandle*/,
				  const char		* /*oldID*/,
				  const char		* /*newID*/ )
{
	return CPI_TRUE;
}


////////////////////////////////////////


static char *
NullTemporaryFileID( CPI_PrivateData	 /*myHandle*/,
					 const char			* /*fileID*/ )
{
	char	*retval = NULL;

	// Allocate size of the existing name + 5 for fileID.tmp
	retval = (char *)cpiAlloc( 10 );

	if ( NULL != retval )
		strcpy( retval, "NULL.tmp" );

	return retval;
}


/////////////////////////////////////////


static RPI_FMInfo fileMgrInfo = {
	{
		"nullfile",
		"NULL File Manager",
		"Silicon Grail",
		"2.2",
		"manual/manual_toc.html",
		NULL
	},

	NULL,		// We don't need to do any initilization
	NULL,		// We don't need to do any shutdown
	NullGetFileIDList,
	NullGetMetadataToOpen,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NullGetDeviceName,
	NullCreateDirID,
	NullRemoveDirID,
	NullRemoveFileID,
	NullRenameFileID,
	NullTemporaryFileID,
	NULL
};


//////////////////////////////////////


static CPI_PrivateData
NullOpen( CPI_Metadata data )
{
	CPI_Bool	 isread, iswrite, istrunc;
	deviceState	*dev	= NULL;

	cpiGetRead( data, &isread );
	cpiGetWrite( data, &iswrite );
	cpiGetTrunc( data, &istrunc );

	if ( CPI_FALSE == iswrite )
	{
		cpiError( "NULL Device only available for write" );
	}
	else
	{
		dev = (deviceState *)cpiAlloc( sizeof( deviceState ) );

		if ( NULL != dev )
		{
			dev->theCurPos	= 0;
			dev->theFileLen	= 0;
			dev->theIsOpen	= CPI_TRUE;
		}
	}

	return (CPI_PrivateData)dev;
}


////////////////////////////////////////////


static void
NullClose( CPI_PrivateData handle )
{
	deviceState *dev = (deviceState *)handle;

	if ( NULL != dev )
	{
		cpiFree( dev );
	}
}


////////////////////////////////////////////


static CPI_Bool
NullIsOpen( CPI_PrivateData handle )
{
	deviceState *dev = (deviceState *)handle;

	return NULL != dev ? dev->theIsOpen : CPI_FALSE;
}


////////////////////////////////////////////


static void
NullSync( CPI_PrivateData /*handle*/ )
{
}


/////////////////////////////////////////////


static CPI_Uint64
NullGetPos( CPI_PrivateData handle )
{
	deviceState *dev = (deviceState *)handle;
	
	return NULL != dev ? dev->theCurPos : 0;
}


/////////////////////////////////////////////


static CPI_Uint64
NullSetPos( CPI_PrivateData handle, CPI_Int64 pos, CPI_Uint8 whence )
{
	CPI_Uint64	 retval	= CPI_Uint64( -1 );
	deviceState *dev	= (deviceState *)handle;

	if ( NULL != dev )
	{
		if ( CPI_SEEK_START == whence )
		{
			if ( pos < 0 )
			{
				dev->theCurPos	= 0;
				retval			= 0;
			}
			else if ( (CPI_Uint64)(pos) > dev->theFileLen )
			{
				if ( 0 < dev->theFileLen )
				{
					dev->theCurPos	= dev->theFileLen - 1;
					retval			= dev->theCurPos;
				}
				else
				{
					dev->theCurPos	= 0;
					retval			= 0;
				}
			}
			else
			{
				dev->theCurPos = pos;
				retval = pos;
			}
		}
		else if ( CPI_SEEK_CUR == whence )
		{
			if ( ( dev->theCurPos + pos ) > dev->theFileLen )
			{
				if ( 0 < dev->theFileLen )
				{
					dev->theCurPos	= dev->theFileLen - 1;
					retval			= dev->theCurPos;
				}
				else
				{
					dev->theCurPos	= 0;
					retval			= 0;
				}
			}
			else
			{
				dev->theCurPos += pos;
				retval			= dev->theCurPos;
			}
		}
		else if ( CPI_SEEK_END == whence )
		{
			if ( 0 < pos )
			{
				if ( 0 < dev->theFileLen )
				{
					dev->theCurPos	= dev->theFileLen - 1;
					retval			= dev->theCurPos;
				}
				else
				{
					dev->theCurPos	= 0;
					retval			= 0;
				}
			}
			else if ( (CPI_Uint64)( - pos ) > dev->theFileLen )
			{
				dev->theCurPos	= 0;
				retval			= 0;
			}
			else if ( 0 < dev->theFileLen )
			{
				dev->theCurPos	= dev->theFileLen + pos - 1;
				retval			= dev->theCurPos;
			}
			else
			{
				dev->theCurPos	= 0;
				retval			= 0;
			}
		}
	}

	return retval;
}


////////////////////////////////////////


static CPI_Bool
NullGetFileLength( CPI_PrivateData handle, CPI_Uint64 *length )
{
	CPI_Bool	 retval = CPI_FALSE;
	deviceState *dev = (deviceState *)handle;

	if ( NULL != dev )
	{
		*length = dev->theFileLen;
		retval	= CPI_TRUE;
	}
	else
		*length = 0;

	return retval;
}


//////////////////////////////////////////////


static CPI_Uint64
NullRead( CPI_PrivateData	 /*handle*/,
		  void				* /*buffer*/,
		  CPI_Uint64		 /*bytecount*/ )
{
	return 0;
}


///////////////////////////////////////////////


static CPI_Uint64
NullWrite( CPI_PrivateData	 handle,
		   const void		* /*buffer*/,
		   CPI_Uint64		 bytecount )
{
	deviceState		*dev	= (deviceState *)handle;

	if ( NULL != dev &&
		 ( dev->theCurPos + bytecount ) < dev->theFileLen )
	{
		dev->theFileLen = dev->theCurPos + bytecount;
	}

	dev->theCurPos += bytecount;

	return bytecount;
}


///////////////////////////////////////////////


static RPI_DeviceInfo deviceInfo = {
	{
		"nulldevice",
		"NULL Device",
		"Silicon Grail",
		"2.2",
		"manual/manual_toc.html",
		NULL
	},

	NullOpen,
	NullClose,
	NullIsOpen,
	NullSync,
	NullGetPos,
	NullSetPos,
	NullGetFileLength,
	NullRead,
	NullWrite
};


//////////////////////////////////////////


CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool retval = true;

	retval =  cpiRegisterDevice( &deviceInfo );

	if ( retval )
		retval = cpiRegisterFileManager( &fileMgrInfo );

	return retval;
}


//////////////////////////////////////////////


CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterFileManager( "nullfile" );
	cpiUnregisterDevice( "nulldevice" );
}
