/*
//
// Produced by:
//  Silicon Grail Corporation
//  736 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:    AddColor
//
// COMMENTS: This adds a color to your image
//
*/

#include <stdio.h>
#include <math.h>
#include <CPI.h>


////////////////////////////////////////


#define MAX8	(CPI_Uint8)255
#define MAX16	(CPI_Uint16)65535
#define RED		0x01
#define GREEN	0x02
#define BLUE	0x04
#define ALPHA	0x08


////////////////////////////////////////


typedef struct _addColorData
{
	CPI_Float32	 color[4];
	CPI_Bool	 selected[4];
} addColorData;


////////////////////////////////////////


static CPI_Float32	 colordefs[] = { 0.0F, 0.0F, 0.0F, 0.0F };

static const char	*maskaction[]	=
{
	"On/Off",
	"Mix",
	"None",
	NULL
};

static const char	*channelmenu[]	=
{
	"Red",
	"Green",
	"Blue",
	"Alpha",
	NULL
};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Node Functions /////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


static CPI_Bool
AddColorInit( void )
{
	cpiAddColorGroup( "color",
					  "Color",
					  colordefs,
					  4,
					  CPI_PARM_ANIMATABLE | CPI_PARM_EXPANDBYDEFAULT,
					  "Color to add to your image",
					  NULL );
	cpiAddChannelSelect( "Select which channels to add color to",
						 NULL );
	cpiAddToggle( "mask_invert",
				  "Mask Invert",
				  CPI_FALSE,
				  CPI_PARM_DEFAULT,
				  "Invert the mask channel?",
				  NULL );
	
	cpiAddMenu( "mask_action",
				"Mask Action",
				1,
				maskaction,
				"Select how mask affects the output image",
				NULL );

	cpiAddMenu( "mask_channel",
				"Mask Channel",
				0,
				channelmenu,
				"Select which channel to use as the mask image",
				NULL );				
	
	return CPI_TRUE;
}


////////////////////////////////////////


static CPI_Bool
AddColorShutDown( void )
{
	return CPI_TRUE;
}


////////////////////////////////////////


static void
AddColorParmChanged( CPI_PrivateData	 /*handle*/,
					 const char			*parmname )
{
	// We only want to check this when we first create the node, otherwise
	// it'll be handled by inputChanged
	if ( NULL == parmname )
	{
		CPI_Bool	 haveMask = cpiIsInputConnected( 1 );

		cpiEnableParameter( "mask_invert", haveMask );
		cpiEnableParameter( "mask_action", haveMask );
		cpiEnableParameter( "mask_channel", haveMask );
	}
}


////////////////////////////////////////


static void
AddColorInputChanged( CPI_PrivateData	/*handle*/,
					  CPI_Uint32		/*inputNum*/ )
{
	CPI_Bool	 haveMask	= cpiIsInputConnected( 1 );

	cpiEnableParameter( "mask_invert", haveMask );
	cpiEnableParameter( "mask_action", haveMask );
	cpiEnableParameter( "mask_channel", haveMask );
}


////////////////////////////////////////


// label the node inputs - this is optional
// Input is the number of the input you want to provide a
// label for, starting at 0 - this label shows up in the
// status line when the user mouses over the connector.
// The first letter of each string is used to label the input
// connector.

static const char *
AddColorInputLabel( CPI_PrivateData    /*handle*/,
					CPI_Uint32         input)
{
	static const char	input1[] = {"input image"};
	static const char	input2[] = {"mask image"};

	if ( input == 0 )
		return input1;
	else
		return input2;
}


////////////////////////////////////////


static CPI_ImageOp
AddColorExec( CPI_PrivateData	 /*handle*/,
			  CPI_Float32		 myTime,
			  CPI_Uint8			 quality,
			  CPI_Uint32	 	 nodeOutput,
			  CPI_Bool			 viewerOutput,
			  CPI_Float32		 scaleX,
			  CPI_Float32		 scaleY )
{
	CPI_Int32		 invertMask;
	CPI_Int32		 chanSel;
	CPI_ImageOp	 	 inputs[2];
	CPI_ImageOp		 colorOut   = NULL;
	CPI_PrivateData	 opData;
	addColorData	*opState	= NULL;
	CPI_ImageOp		 retval		= NULL;

	opData		= cpiCreatePrivateData( "addcolor" );
	opState		= (addColorData *)opData;

	if ( NULL != opState )
	{
		if ( cpiGetFloat( &opState->color[0], "color.red", myTime ) &&
			 cpiGetFloat( &opState->color[1], "color.green", myTime ) &&
			 cpiGetFloat( &opState->color[2], "color.blue", myTime ) &&
			 cpiGetFloat( &opState->color[3], "color.alpha", myTime ) &&
			 cpiGetInteger( &invertMask, "mask_invert", myTime ) &&
			 cpiGetChannelSelect( &chanSel ) )
		{
			opState->selected[0] = ( chanSel & RED ) != 0;
			opState->selected[1] = ( chanSel & GREEN ) != 0;
			opState->selected[2] = ( chanSel & BLUE ) != 0;
			opState->selected[3] = ( chanSel & ALPHA ) != 0;
			
			inputs[0]	= cpiGetInputOp( 0, myTime, quality, scaleX, scaleY );

			if ( cpiIsInputConnected( 1 ) )
				inputs[1]	= cpiGetInputOp( 1, myTime,
											 quality, scaleX, scaleY );
			else
				inputs[1] = NULL;
			
			if ( NULL != inputs[0] )
				colorOut = cpiAddImageOp( "addcolor", opState, &inputs[0], 1 );
			else
				cpiError( "Missing required input" );

			if ( NULL != inputs[1] )
			{
				CPI_ImageOp		 maskInputs[3];
				CPI_PrivateData	 maskData	= NULL;
				maskParms		*maskPrms	= NULL;

				if ( invertMask )
				{
					CPI_PrivateData	 invertData = NULL;
								
					invertData	= cpiCreatePrivateData( "invert" );

					if ( NULL != invertData )
						inputs[1] = cpiAddImageOp( "invert",
													 invertData,
													 &inputs[1],
													 1 );
				}

				maskInputs[0]	= colorOut;
				maskInputs[1]	= inputs[0];
				maskInputs[2]	= inputs[1];
				
				maskData 	= cpiCreatePrivateData( "mask" );
				maskPrms	= (maskParms *)maskData;

				if ( NULL != maskPrms )
				{
					CPI_Int32	maskAction;
					
					if ( cpiGetInteger( &maskAction,
										"mask_action",
										myTime ) &&
						 cpiGetInteger( &maskPrms->maskChannel,
										"mask_channel",
										myTime ) )
					{
						maskPrms->maskType = (CPI_MaskType)maskAction;

						retval = cpiAddImageOp( "mask",
												maskData,
												maskInputs, 3 );
					}
					else
						cpiError( "Couldn't get mask parms" );
										
				}
				else
					cpiError( "Couldn't get mask input private data" );
			}
			else
				retval = colorOut;
		}
		else
			cpiError( "Couldn't get AddColor parameter(s)" );
	}

	return retval;
}


////////////////////////////////////////


static RPI_NodeInfo ninfo = {
	{
		"addcolor",
		"Add Color",
		"Silicon Grail",
		"2.2",
	},
	AddColorInit,
	AddColorShutDown,
	NULL,
	NULL,
	AddColorParmChanged,
	NULL,
	AddColorInputChanged,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	AddColorExec,
	AddColorInputLabel,
	NULL,
	NULL,
	NULL,
	1,
	2,
	1,
	1,
	CPI_FALSE
};


//////////////////////////////////////////////////////////////////////////
// Image Op Functions ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

// this templated version is used for 8 and 16 bit color space

template <class T> static
void add_color( CPI_Image	&input,
				CPI_Image	*result,
				T			 add[],
				CPI_Bool	 selected[],
				T			 max )
{
	CPI_Float32	 val;
	CPI_Int32	 x, y, endY, c;
	CPI_Int32	 channels	= result->myContext.channels;
	T			*pin, *pout;

	endY = result->myContext.offsetY + result->myContext.sizeY;

	for ( y = result->myContext.offsetY; y < endY; ++y )
	{
		pin		= (T *)cpiGetLine( &input, y );
		pout	= (T *)cpiGetLine( result, y );

		for ( x = 0; x < result->myContext.sizeX; ++x )
		{
			for ( c = 0; c < channels; ++c )
			{
				if ( selected[c] )
				{
					val =  *pin++ + add[c];

					if ( val > max )
						*pout++ = (T)max;
					else
						*pout++ = (T)val;
				}
				else
					*pout++ = *pin++;
			}
		}
	}
}


// The float version is separate because we don't want to clip values in
// float space

static
void add_colorF( CPI_Image	   &input,
			 	 CPI_Image	   *result,
			 	 CPI_Float32	add[],
				 CPI_Bool	 	selected[] )
{
	CPI_Int32	 x, y, endY, c;
	CPI_Int32	 channels	= result->myContext.channels;
	CPI_Float32	*pin, *pout;

	endY = result->myContext.offsetY + result->myContext.sizeY;

	for ( y = result->myContext.offsetY; y < endY; ++y )
	{
		pin		= (CPI_Float32 *)cpiGetLine( &input, y );
		pout	= (CPI_Float32 *)cpiGetLine( result, y );

		for ( x = 0; x < result->myContext.sizeX; ++x )
		{
			for ( c = 0; c < channels; ++c )
			{
				if ( selected[c] )
				{
					*pout++ =  *pin++ + add[c];
				}
				else
					*pout++ = *pin++;
			}
		}
	}
}


////////////////////////////////////////


static CPI_PrivateData
AddColorCreateOpInstance( void )
{
	addColorData *retval = (addColorData *)cpiAlloc( sizeof( addColorData ) );

	return (CPI_PrivateData)retval;
}


////////////////////////////////////////


static CPI_Bool
AddColorDestroyOpInstance( CPI_PrivateData handle )
{
	CPI_Bool	 	 retval = CPI_FALSE;
	addColorData	*state	= (addColorData *)handle;

	if ( NULL != state )
	{
		cpiFree( state );
		retval = CPI_TRUE;
	}

	return retval;
}


////////////////////////////////////////


static CPI_Bool
AddColorProcess( CPI_PrivateData	 handle,
				 CPI_Image			*result,
				 CPI_Image			 inputs[],
				 CPI_Uint32			 numInputs )
{
	CPI_Int32		 i;
	CPI_Bool		 retval = CPI_FALSE;
	addColorData	*state	= (addColorData *)handle;

	if ( NULL != state )
	{
		retval = CPI_TRUE;
		
		if ( result->myContext.bitsPerPel == 8 )
		{
			CPI_Uint8	color[4];

			for ( i = 0; i < 4; ++i )
				color[i]	= cpiRound( state->color[i] * MAX8 );

			add_color( inputs[0], result, color, state->selected,
					   (CPI_Uint8)MAX8 );
		}
		else if ( result->myContext.bitsPerPel == 16 )
		{
			CPI_Uint16	color[4];

			for ( i = 0; i < 4; ++i )
				color[i] = cpiRound( state->color[i] * MAX16 );

			add_color( inputs[0], result, color, state->selected,
					   (CPI_Uint16)MAX16 );
		}
		else if ( result->myContext.bitsPerPel == 32 )
		{
			add_colorF( inputs[0], result, state->color, state->selected );
		}
		else
		{
			cpiError( "Add Color got an invalid result bit depth" );
			retval = CPI_FALSE;
		}
	}
	else
		cpiError( "Couldn't get Add Color parameter(s)" );

	return retval;
}


////////////////////////////////////////


static RPI_ImageOpInfo opinfo =
{
	{
		"addcolor",
		"",
		"Silicon Grail",
		"2.2"
	},
	AddColorCreateOpInstance,
	AddColorDestroyOpInstance,
	AddColorProcess,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	1,
	CPI_TRUE,
	CPI_FALSE
};


////////////////////////////////////////


CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool	retval = CPI_TRUE;

	retval = cpiRegisterImageOp( &opinfo );

	if ( retval )
		retval = cpiRegisterNode( &ninfo );

	return retval;
}


////////////////////////////////////////


CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterImageOp( "addcolor" );
	cpiUnregisterNode( "addcolor" );
}

				 
