/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	ImgDesc
//
// COMMENTS: This prints out the contents of the image description
// 			 structure, as a way of demonstrating how to get the
//			 context and what it means.
//
*/

#include <stdio.h>
#include <unistd.h>
#include <CPI.h>

//////////////////////////////////////


static CPI_Bool
ImgDescInit( void )
{
	return CPI_TRUE;
}


//////////////////////////////////////


static CPI_Bool
ImgDescShutDown( void )
{
	return CPI_TRUE;
}


//////////////////////////////////////


CPI_ImageOp
ImgDescExec( CPI_PrivateData	 /*handle*/,
			 CPI_Float32	 	 myTime,
			 CPI_Uint8		 	 quality,
			 CPI_Uint32		 	 output,
			 CPI_Bool		 	 viewerOutput,
			 CPI_Float32	 	 scaleX,
			 CPI_Float32	 	 scaleY )
{
	CPI_ImageOp	retval	= NULL;
	CPI_ImageOp	inputOp = cpiGetInputOp( 0, myTime, quality, scaleX, scaleY );

	if ( NULL != inputOp )
		retval = cpiAddImageOp( "imgdesc", NULL, &inputOp, 1 );

	return retval;
}


//////////////////////////////////////


static RPI_NodeInfo ninfo = {
	{
		"imgdesc",
		"Image Description",
		"Silicon Grail",
		"2.2"
	},
	ImgDescInit,
	ImgDescShutDown,
	NULL,		// CreateInstance
	NULL,		// DestroyInstance
	NULL,		// ParmChanged
	NULL,		// InputAdded
	NULL,		// InputChanged
	NULL,		// InputRemoved
	NULL,		// GetRegion
	NULL,		// GetFullSize
	NULL,		// GetRange
	NULL,		// GetErrors
	ImgDescExec,
	NULL,		// InputLabel
	NULL,		// OutputLabel
	NULL,		// ViewerLabel
	NULL,		// NumViewerOutputs
	1,
	1,
	1,
	1,
	CPI_FALSE
};


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Image Op Functions /////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


template <class T>
void copy_image( CPI_Image	*result,
				 CPI_Image	*input,
				 T			 max )
{
	CPI_Uint32	 endX, endY;
	T			*out	= (T *)result->data;
	T			*in		= (T *)input->data;

	endX = result->myContext.offsetX + result->myContext.sizeX;
	endY = result->myContext.offsetY + result->myContext.sizeY;

	for ( CPI_Uint32 y = result->myContext.offsetY; y < endY; ++y )
	{
		for ( CPI_Uint32 x = result->myContext.offsetX; x < endX; ++x )
		{
			for ( CPI_Int32 c = 0; c < result->myContext.channels; ++c )
				*out++ = *in++;
		}
	}
}


///////////////////////////////////////////


CPI_Bool
ImgDescProcess( CPI_PrivateData	 /*handle*/,
				CPI_Image		*result,
				CPI_Image	 	 inputs[],
				CPI_Uint32	 	 numInputs )
{
	CPI_Bool retval = CPI_FALSE;

	if ( NULL != result && numInputs == 1 )
	{
		CPI_ImageContext	c = inputs[0].myContext;

		retval = CPI_TRUE;

		printf( "Image Description : \n" );
		printf( "\tsize X		= %d\n", c.sizeX );
		printf( "\tsize Y		= %d\n", c.sizeY );
		printf( "\toffset X	= %d\n", c.offsetX );
		printf( "\toffset Y	= %d\n", c.offsetY );
		printf( "\tchannels	= %d\n", c.channels );
		printf( "\tbitdepth	= %d\n", c.bitsPerPel );
		printf( "\tpixelRatio	= %f\n", c.pixelRatio );
		if ( 0 == c.dataType )
			printf( "\tdataType	= LINEAR\n" );
		else if ( 1 == c.dataType )
			printf( "\tdataType	= LOG\n" );
		else if ( 2 == c.dataType )
			printf( "\tdataType	= VIDEO\n" );
		printf( "\trefBlack	= %f\n", c.referenceBlack );
		printf( "\trefWhite	= %f\n", c.referenceWhite );
		printf( "\tlinearWhite	= %f\n", c.linearWhite );
		printf( "\tdisplayGamma	= %f\n", c.displayGamma );
		printf( "\tfilmGamma	= %f\n", c.filmGamma );
		printf( "\tvideoGamma	= %f\n", c.videoGamma );
		printf( "\tpremult		= %d\n", c.isPremultiplied );
		printf( "\tbacking r	= %f\n", c.backingRed );
		printf( "\tbacking g	= %f\n", c.backingGreen );
		printf( "\tbacking b	= %f\n", c.backingBlue );
		printf( "\tbacking a	= %f\n", c.backingAlpha );

		if ( c.bitsPerPel == 8 )
			copy_image( result, &inputs[0], (CPI_Uint8)255 );
		else if ( c.bitsPerPel == 16 )
			copy_image( result, &inputs[0], (CPI_Uint16)65535 );
		else if ( c.bitsPerPel == 32 )
			copy_image( result, &inputs[0], (CPI_Float32)1.0 );
		else
		{
			cpiError( "Inputs image has an invalid bit depth!" );
			retval = CPI_FALSE;
		}
	}

	return retval;
}


//////////////////////////////////////


static RPI_ImageOpInfo opinfo =
{
	{
		"imgdesc",
		"Image Description",
		"Silicon Grail",
		"2.2"
	},
	NULL,
	NULL,
	ImgDescProcess,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	1,
	CPI_FALSE,
	CPI_FALSE
};
	

//////////////////////////////////////


CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool	retval = CPI_TRUE;

	retval = cpiRegisterImageOp( &opinfo );

	if ( retval )
		retval = cpiRegisterNode( &ninfo );

	return retval;
}


///////////////////////////////////////


CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterImageOp( "imgdesc" );
	cpiUnregisterNode( "imgdesc" );
}
