/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	Pizza
//
// COMMENTS: Select and order a pizza via email
//
*/

#include <stdio.h>
#include <math.h>
#include <CPI.h>

#include "Pizza.h"

///////////////////////////////////////////

static const char *sizemenu[] =
{
	"small",
	"medium",
	"large",
	"slice",
	NULL
};

static const char *crustmenu[] =
{
	"thin",
	"thick",
	NULL
};

static struct topping
{
	const char 	*code;
	const char 	*label;
	CPI_Bool	 wantIt;
} toppings[] =
{
	{ "pep",	"Pepperoni",		CPI_FALSE },
	{ "mush",	"Mushrooms",		CPI_FALSE },
	{ "gpep",	"Green Peppers",	CPI_FALSE },
	{ "saus",	"Sausage",			CPI_FALSE },
	{ "olive",	"Olives",			CPI_FALSE },
	{ "xchese",	"Extra Cheese",		CPI_FALSE },
};

#define NTOPPINGS (CPI_Int32)( sizeof(toppings) / sizeof(struct topping) )

static void
submit( const char 		*/*nodename*/,
		const char 		*/*parmname*/,
		CPI_Float32		 time,
		CPI_PrivateData	 /*data*/ )
{
	CPI_Int32	i, size, crust;
	char		strval[100], name[50];
	CPI_Bool	oneOut = CPI_FALSE;

	cpiGetString( strval, 100, "order", time );
	cpiGetInteger( &size, "size", time );
	cpiGetInteger( &crust, "crust", time );
	for ( i = 0; i < NTOPPINGS; i++ )
	{
		sprintf( name, "toppings.%s", toppings[i].code );
		cpiGetInteger( &toppings[i].wantIt, name, time );
	}

	printf("Order to: %s\n", strval);
	printf("Please deliver a %s, %s crust pizza\n", sizemenu[size], 
			crustmenu[crust]);
	printf("Extra Toppings:\n");
	for (i = 0; i < NTOPPINGS; i++ )
	{
		if ( toppings[i].wantIt == CPI_TRUE )
		{
			printf("\t%s\n", toppings[i].label );
			oneOut = CPI_TRUE;
		}
	}

	if ( oneOut == CPI_FALSE )
		printf("\tNone\n");
}


static CPI_Bool
PizzaInit( void )
{
	CPI_Int32	i;
	char		helpstring[128];

	cpiAddString( "order", "Send to", "orders@bigfatpizza.com",
				  CPI_FALSE, CPI_PARM_DEFAULT,
				  "Email address to send order to", NULL );

	cpiAddMenu( "size", "Size", 0, sizemenu,
				"Choose the size of the pizza", NULL );

	cpiAddMenu( "crust", "Crust", 0, crustmenu,
				"Choose the type of crust", NULL );

	cpiStartGroup( "toppings", "Toppings", CPI_PARM_DEFAULT,
				   "This folder holds choices for toppings", NULL );
	for ( i = 0; i < NTOPPINGS; i++ )
	{
		sprintf( helpstring,  "Add %s to the pizza", toppings[i].label );
		cpiAddToggle( toppings[i].code, toppings[i].label, 
					  CPI_FALSE, CPI_PARM_DEFAULT, helpstring, NULL );
	}
	cpiEndGroup( );

	cpiAddButton( "submit", "Send Order", submit,
				  CPI_PARM_DEFAULT,
				  "Send email ordering the pizza", NULL );

	// set up the pizza icon
	cpiRegisterIconData( "Pizza.tiff", Pizza_SIZEX, Pizza_SIZEY, 
						  Pizza_data, 255, 0, 255 );

	// define the name of the pizza icon
	cpiDefineIcon( "Nodes/Pizza", "Pizza.tiff",
				   NULL, NULL, NULL, NULL, NULL, 255, 0, 255 );

	return CPI_TRUE;
}


///////////////////////////////////////////


CPI_Bool
PizzaShutDown( void )
{
	return CPI_TRUE;
}



//////////////////////////////////////////


CPI_ImageOp
PizzaExec( CPI_PrivateData	/* handle */,
		   CPI_Float32		myTime,
		   CPI_Uint8		quality,
		   CPI_Uint32		/*output*/,
		   CPI_Bool			/*viewerOutput*/,
		   CPI_Float32		scaleX,
		   CPI_Float32		scaleY )
{
	CPI_ImageOp	 retval	= NULL;

	// just return the input image
	retval = cpiGetInputOp( 0, myTime, quality, scaleX, scaleY );
		
	return retval;
}


////////////////////////////////////////////

static RPI_NodeInfo ninfo = {
	{
		"pizza",
		"Pizza",
		"Silicon Grail",
		"2.2", NULL, NULL
	},
	PizzaInit,
	PizzaShutDown,
	NULL,		// Create Instance
	NULL,		// Destroy Instance
	NULL,		// Parm Changed
	NULL,		// Input Added
	NULL,		// InputChanged
	NULL,		// Input Removed
    NULL,		// Get Region
    NULL,		// Get Full Size
	NULL,		// PizzaGetRange,
	NULL,		// Get Errors
	PizzaExec,
	NULL,		// InputLabel,
	NULL,		// OutputLabel,
	NULL,		// ViewerOutputLabel
	NULL,		// NumViewerOutputs
	1, 
	1,
	1,
	1,
	CPI_FALSE
};


//////////////////////////////////////////


CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool retval = CPI_TRUE;

	retval = cpiRegisterNode( &ninfo );

	return retval;	
}


//////////////////////////////////////////////


CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterNode( "pizza" ); 
}
