/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	Toolbar demo
//
// COMMENTS:
//
*/

#include <stdio.h>
#include <math.h>
#include <CPI.h>
#include "../Nodes/Widgets.h"


////////////////////////////////////////


typedef struct _toolbarState
{
	CPI_Int32		intvalue;
} toolbarState;


typedef struct _overlayState
{
	CPI_Int32		 fullSizeX;
	CPI_Int32		 fullSizeY;
	CPI_Int32		 operation;
	CPI_Float32		 theTime;
} overlayState;


////////////////////////////////////////


static const char	*animals[]		=
{
	"Lion",
	"Tiger",
	"Bear",
	NULL,
};


static const char	*items[]		=
{
	"Yes",
	"No",
	"Maybe so",
	NULL,
};

///////////////////////////////////////////////////////////////////////////////
// Node Functions /////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


static CPI_Bool
ToolbarInit( void )
{			   
	cpiAddString( "text", "Feedback", "Click in toolbar", CPI_FALSE,
				  CPI_PARM_DEFAULT ,
				  "Show the toolbar widget and value", NULL );
	return CPI_TRUE;
}


////////////////////////////////////////


CPI_ImageOp
ToolbarExec( CPI_PrivateData	 handle,
			  CPI_Float32	 	 myTime,
			  CPI_Uint8		 	 quality,
			  CPI_Uint32		 output,
			  CPI_Bool		 	 viewerOutput,
			  CPI_Float32	 	 scaleX,
			  CPI_Float32	 	 scaleY )
{
	CPI_ImageOp		 retval		= NULL;
	
	retval = cpiGetInputOp( 0, myTime, quality, scaleX, scaleY );
	
	return retval;
}


////////////////////////////////////////


static RPI_NodeInfo ninfo = {
	{
		"toolbar",
		"Toolbar",
		"Silicon Grail",
		"v1.0"
	},
	ToolbarInit,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	ToolbarExec,
	NULL,
	NULL,
	NULL,
	NULL,
	1,
	1,
	1,
	1,
	CPI_FALSE
};


///////////////////////////////////////////////////////////////////////////////
// Overlay Functions //////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


CPI_Bool
ToolbarOverlayInit( void )
{
	// To define a toolbar, first declare one open
	cpiStartToolbar();

	// Now add GUI widgets just as you would for node parameters
	// Available widgets currently are:
	// Toggle & Icon Toggle
	// Button & Icon Button
	// Button Strip & Icon Button Strip
	// Menu

	// define a menu
	cpiAddMenu( "results", "Results", 0, items, 
				"Choose a result from the menu", NULL );
	
	// define a button strip
	cpiAddButtonStrip( "animal", "Animal", 1, CPI_PARM_DEFAULT,
					   animals, "Choose an animal mode", NULL );

	// here we define an icon toggle
	// See the node example Widgets.C for details
	cpiRegisterIconData( "toolbar_Off", 24, 18, off_data, 255, 0, 255 );
    cpiRegisterIconData( "toolbar_On", 24, 18, on_data, 255, 0, 255 );
    cpiDefineIcon( "toolbar_Off", "toolbar_Off",
                   NULL, NULL,
                   "toolbar_Off", NULL, NULL, 255, 0, 255 );
    cpiAddIconToggle( "itoggle", "toolbar_Off", CPI_FALSE, CPI_PARM_DEFAULT,
                      "Toggle button with icon", NULL );

	// then close the toolbar, name it, and attach it to an overlay
	cpiEndToolbar( "toolbarstrip",				// name of overlay
				   "Toolbar Strip",				// label
				   CPI_WINDOW_BOTTOM,			// default alignment
				   CPI_TRUE,					// allow horizontal layout?
				   CPI_TRUE );					// allow vertical layout?
	
	return CPI_TRUE;
}


////////////////////////////////////////


static CPI_PrivateData
ToolbarCreateOverlayInstance( void )
{
	overlayState *retval = (overlayState *)cpiAlloc( sizeof( overlayState ) );

	if ( NULL != retval )
	{
		retval->fullSizeX	= 720;
		retval->fullSizeY	= 486;
	}
	
	return retval;
}


////////////////////////////////////////


static void
ToolbarDestroyOverlayInstance( CPI_PrivateData data )
{
	overlayState 	*state	= (overlayState *)data;

	if ( NULL != state )
		cpiFree( state );
}


////////////////////////////////////////


static void
ToolbarSourceChanged( CPI_PrivateData	 handle,
					   CPI_ImageContext	 fullSize,
					   CPI_Float32		 thetime )
{
	overlayState *state = (overlayState *)handle;
	
	if ( NULL != state )
	{
		state->fullSizeX 	= fullSize.sizeX;
		state->fullSizeY 	= fullSize.sizeY;
		state->theTime		= thetime;
	}
}


////////////////////////////////////////


static void
ToolbarParmChanged( CPI_PrivateData	 handle,
				    const char		*parmname )
{
	overlayState *state = (overlayState *)handle;
	CPI_Int32	  choice;
	char	  	  newstring[100];

	if ( NULL != state &&
		 cpiGetInteger( &choice, parmname, 0.0F ) )
	{
		sprintf(newstring, "parameter %s,  choice = %d\n", parmname, choice );
		cpiSetString( newstring, "text", 0.0F, CPI_FALSE );
	}
}


////////////////////////////////////////


static void
ToolbarRender( CPI_PrivateData /* handle */,
			   CPI_Bool		   /* isPicking */ )
{
}


////////////////////////////////////////


static RPI_OverlayInfo oinfo = {
	{
		"toolbar",
		"Toolbar",
		"Silicon Grail",
		"2.2"
	},
	ToolbarOverlayInit,
	NULL,
	ToolbarCreateOverlayInstance,
	ToolbarDestroyOverlayInstance,
	ToolbarSourceChanged,
	ToolbarSourceChanged,
	ToolbarParmChanged,
	ToolbarSourceChanged,
	ToolbarRender,				// required function
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	"toolbar",
	CPI_TRUE
};



////////////////////////////////////////


CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool	 retval = CPI_TRUE;

	retval = cpiRegisterNode( &ninfo );

	if ( retval )
		retval = cpiRegisterOverlay( &oinfo );

	return retval;
}


////////////////////////////////////////


CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterNode( "toolbar" );
	cpiUnregisterOverlay( "toolbar" );
}
