/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_FileImporter
//
// COMMENTS:
//
*/

#ifndef _CPI_FileImporter_h_
#define _CPI_FileImporter_h_

#include <CPI/CPI_Types.h>

#ifdef __cplusplus
extern "C" {
#endif

/* ///////////////////////////////
   //  Node Reading Functions   //
   /////////////////////////////// */


CPIDSOIMPORT void			 cpiStartNode( const char	*name,
										   const char	*type,
										   const char	*inputs[] );

CPIDSOIMPORT void			 cpiEndNode( void );

CPIDSOIMPORT void			 cpiGetNodeName( char		*buffer,
											 CPI_Uint32	 bufferSize );
	
CPIDSOIMPORT void			 cpiDisableNode( void );

CPIDSOIMPORT void			 cpiCloneNode( const char	*from );
	
CPIDSOIMPORT void			 cpiExportInput( const char	*childNode,
											 CPI_Int32	 inputNum );
CPIDSOIMPORT void			 cpiExportOutput( const char	*childNode,
											  CPI_Int32		 outputNum );
CPIDSOIMPORT void			 cpiImportParm( const char	*childNode,
											const char	*oldName,
											const char	*newName );

CPIDSOIMPORT void			 cpiSetNodeLocation( CPI_Float32	 x,
												 CPI_Float32	 y );
CPIDSOIMPORT void			 cpiSetNodeSize( CPI_Float32	x,
											 CPI_Float32	y );
CPIDSOIMPORT void			 cpiSetNodeThumbnail( CPI_Bool	 onoff );
CPIDSOIMPORT void			 cpiSetNodeIcon( CPI_Bool	onoff );
CPIDSOIMPORT void			 cpiSetNodeVisibleInTimeline( CPI_Bool	onoff );
CPIDSOIMPORT void			 cpiSetNodeAutoKeyState( CPI_Bool onoff );

	
/* Parm creation/modification functions */
CPIDSOIMPORT void			 cpiCreateSpareInt( const char	*name,
												CPI_Int32	 def,
												CPI_Int32	 min,
												CPI_Int32	 max );
CPIDSOIMPORT void			 cpiCreateSpareFloat( const char	*name,
												  CPI_Float32	 def,
												  CPI_Float32	 min,
												  CPI_Float32	 max );
CPIDSOIMPORT void			 cpiCreateSpareString( const char	*name,
												   const char	*def );
CPIDSOIMPORT void			 cpiStartParm( const char	*name );
CPIDSOIMPORT void			 cpiEndParm( void );
CPIDSOIMPORT void			 cpiEnableParmFlag( const char	*flag );
CPIDSOIMPORT void			 cpiDisableParmFlag( const char	*flag );	
CPIDSOIMPORT void			 cpiSetParmInt( CPI_Int32 value );
CPIDSOIMPORT void			 cpiSetParmFloat( CPI_Float32	 value );
CPIDSOIMPORT void			 cpiSetParmString( const char	*value );
CPIDSOIMPORT void			 cpiSetParmShape( CPI_Float32	 values[],
											  CPI_Int32		 numOfValues );

/* Keypoint functions */
CPIDSOIMPORT void			 cpiAddIntKeypoint( CPI_Float32		 time,
												CPI_Int32		 value );
CPIDSOIMPORT void			 cpiAddFloatKeypoint( CPI_Float32	 time,
												  CPI_Float32	 value );
CPIDSOIMPORT void			 cpiAddStringKeypoint( CPI_Float32	 time,
												   const char	*value );
CPIDSOIMPORT void			 cpiAddShapeKeypoint( CPI_Float32	 time,
												  CPI_Float32	 values[],
												  CPI_Int32		 numOfValues );

/* Interp functions */
CPIDSOIMPORT void			 cpiSetBezierInterp( CPI_Int32	 	 interpNum,
												 CPI_Float32	 bezierX1,
												 CPI_Float32	 bezierY1,
												 CPI_Float32	 bezierX2,
												 CPI_Float32	 bezierY2 );
CPIDSOIMPORT void			 cpiSetConstantInterp( CPI_Int32	 interpNum );
CPIDSOIMPORT void			 cpiSetCubicInterp( CPI_Int32		 interpNum,
												CPI_Float32		 inSlope,
												CPI_Float32		 outSlope );
CPIDSOIMPORT void			 cpiSetEaseInterp( CPI_Int32	 interpNum );
CPIDSOIMPORT void		 	 cpiSetEaseInInterp( CPI_Int32	 interpNum );
CPIDSOIMPORT void			 cpiSetEaseOutInterp( CPI_Int32	 interpNum );
CPIDSOIMPORT void			 cpiSetExpressionInterp( CPI_Int32	 interpNum,
													 const char	*expression );
CPIDSOIMPORT void			 cpiSetLinearInterp( CPI_Int32	 interpNum );
CPIDSOIMPORT void			 cpiSetQuinticInterp( CPI_Int32	 	 interpNum,
												  CPI_Float32	 inSlope,
												  CPI_Float32	 outSlope,
												  CPI_Float32	 inAccel,
												  CPI_Float32	 outAccel );
/* Time range functions */
CPIDSOIMPORT void			 cpiSetTimeRange( CPI_Float32	 start,
											  CPI_Float32	 end );
CPIDSOIMPORT void			 cpiSetCurrentTime( CPI_Float32	 currentTime );

/* Preference functions */
CPIDSOIMPORT void			 cpiSetPrefInt( const char	*prefName,
											CPI_Int32	 value );
CPIDSOIMPORT void			 cpiSetPrefFloat( const char	*prefName,
											  CPI_Float32	 value );
CPIDSOIMPORT void			 cpiSetPrefString( const char	*prefName,
											   const char	*value );

/* Windowing functions */
CPIDSOIMPORT void			 cpiStartView( const char	*name,
										   const char	*type,
										   CPI_Int32	 posx,
										   CPI_Int32	 posy,
										   CPI_Int32	 sizex,
										   CPI_Int32	 sizey,
										   CPI_Int32	 screen );
CPIDSOIMPORT void			 cpiEndView( void );

CPIDSOIMPORT void			 cpiCreateMenuBar( const char	*name,
											   const char	*type,
											   const char	*side );
#ifdef __cplusplus	
}
#endif


#endif
