/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_ImageOpProvider
//
// COMMENTS:
//
*/

#ifndef _CPI_ImageOpProvider_h_
#define _CPI_ImageOpProvider_h_

#include <CPI/CPI_Base.h>


/* /////////////////////////////////////// */


#ifdef __cplusplus
extern "C" {
#endif

	
/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//  None
//
// Return Value:
//  CPI_PrivateData - This user defined structure is opaque to rayz, but should
//					  contain any information which will be needed when
//					  processing your image operation.  If you couldn't create
//					  your structure properly, return NULL.
//
// Undefined behavior:
//  If this function is not provided then NULL private data will be sent into
//  all of your image op functions
//
*/
typedef CPI_PrivateData		 (*UPI_CreateOPInstanceFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//  CPI_PrivateData - User Defined structure containing imageop specific
//					  information.  NOTE: If you cpiAlloc'ed this pointer
//					  in createInstance, it should be free'd here
//
// Return Value:
//  CPI_Bool - should return CPI_TRUE if the instance was destroyed
//			   successfully
//
// Undefined behavior:
//  This function should be provided if you've provided a createInstance
//	function.
//
*/
typedef CPI_Bool			 (*UPI_DestroyOPInstanceFn)( CPI_PrivateData	handle );	

	
/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Image *result - This is your result image, its data pointer is pointing
//						to the first valid pixel in the data you need to fill
//						in.  This is signified by the offsetX and offsetY
//						fields..
//	CPI_Image inputs[] - This is an array of the input images, if there are
//						 no inputs it will be NULL.
//	CPI_Uint32 numinputs - Number of entries contained in the inputs[] array.
//  CPI_PrivateData handle - User Defined structure containing imageop
//							 specific information.
// Return Value:
//	CPI_Bool - return whether the image processing was successful
*/
typedef CPI_Bool			 (*RPI_ProcessImageFn)( CPI_PrivateData	 handle,
													CPI_Image		*result,
													CPI_Image	 	 inputs[],
													CPI_Uint32	 	 numInputs);


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_ImageContext *input - This is the structure which needs to be filled
//							  out.  It needs the region which you need to
//							  recieve from your input to complete your
//							  processing successfully.
//	CPI_ImageContext output - This is the region which your output has
//							  requested
//	CPI_Uint32 whichInput - This is which input we're currently requesting the
//							required input region for..
//  CPI_PrivateData handle - User Defined structure containing imageop
//							 specific information.
//
// Return Value:
//	CPI_Bool - return whether the input region was obtained successfully
//
// Undefined behavior:
//	If this function is not provided then the input region will be assumed to
//	be the same as the output region.
*/
typedef CPI_Bool			 (*UPI_InputRegionFn)( CPI_PrivateData	 handle,
												   CPI_ImageContext *input,
												   CPI_ImageContext	 output,
												   CPI_Uint32 		 whichInput );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_ImageContext *output - This is the structure which needs to be filled
//							   out.  It needs the region which will be computed
//							   given the inputs and parms.
//	CPI_ImageContext inputs[] - This is an array of the regions which your
//								inputs have.
//	CPI_Uint32 numinputs - Number of inputs contained in the array 'inputs'
//  CPI_PrivateData handle - User Defined structure containing imageop
//							 specific information.
//
// Return Value:
//	CPI_Bool - return whether the output region was obtained successfully
//
// Undefined behavior :
//	If this function is not provided the the output region will be assumed to
//	be the same as the region of your first input.
*/
typedef CPI_Bool		     (*UPI_OutputRegionFn)( CPI_PrivateData   handle,
													CPI_ImageContext *output,
													CPI_ImageContext  inputs[],
													CPI_Uint32	      numInputs );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Image *result - This is your result image, it's data pointer is
//						pointing to the first valid pixel in the data you need
//						to fill in.  This is signified by the offsetX and
//						offsetY fields..
//	CPI_Image inputs[] - This is an array of the inputs image, if there are
//						 no inputs it will be NULL.
//	CPI_Uint32 numinputs - Number of entreies contained in the inputs[] array.
//  CPI_PrivateData handle - User Defined structure containing imageop
//							 specific information.
//
// Return Value:
//	CPI_Bool - return whether the image processing control was successful
//
// Undefined behavior :
//	If this function is not provided then the processing control will use
//	the default processing mechanism, which is:
//  If rpiMultiProcCapable == CPI_TRUE, your rpiProcessImage function will
//  be called 'n' times, each time with a separate piece of the image.
//  Else rpiProcessImage will be called once with the full image.
//
*/
typedef CPI_Bool			 (*UPI_ControlProcessingFn)( CPI_PrivateData handle,
														 CPI_Image	*result,
														 CPI_Image	 inputs[],
														 CPI_Uint32	 numInputs );

  
/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value :
//	CPI_Bool - return whether your image operation was setup successfully
//
// Undefined behavior :
//	If this function is not provided then no setup will be done for your
//	image op
//
*/
typedef CPI_Bool			(*UPI_SetupProcessingFn)( CPI_PrivateData handle );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	bool execSuccess - returns whether the execution you're cleaning up after
//					   was successful or not
//
// Return Value :
//	CPI_Bool - return whether your image operation was cleaned up successfully
//
// Undefined behavior :
//	If this function is not provided then no cleanup will be done for your
//	image op
//
*/
typedef CPI_Bool			(*UPI_CleanupProcessingFn)( CPI_PrivateData handle,
														CPI_Bool 		execSuccess );


/*
// For any optional (upi labeled) functions you don't wish to provide,
// fill in NULL
*/
typedef struct
{
	RPI_PluginInfo			rpiPluginInfo;

	// Create and Destroy private data
	UPI_CreateOPInstanceFn	upiCreateInstance;
	UPI_DestroyOPInstanceFn	upiDestroyInstance;
	
	// Image processing functions
	RPI_ProcessImageFn		rpiProcessImage;
	UPI_InputRegionFn		upiInputRegion;
	UPI_OutputRegionFn		upiOutputRegion;
	UPI_ControlProcessingFn	upiControlProcessing;
	UPI_SetupProcessingFn	upiSetupProcessing;
	UPI_CleanupProcessingFn	upiCleanupProcessing;

	CPI_Uint32				rpiRequiredInputs;
	CPI_Bool				rpiMultiProcCapable; // Can this plugin multi-proc?
	CPI_Bool				rpiSupportDisjointRegions;
} RPI_ImageOpInfo;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_ImageOpProvider_h_ */
