/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
*/

#ifndef _CPI_InternalOps_h_
#define _CPI_InternalOps_h_

#include <CPI/CPI_Types.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
	BOX,
	LINEAR,
	QUADRATIC,
	CUBIC,
	GAUSSIAN,
	ERODE,
	DILATE
} CPI_KernelShape;

typedef enum
{
	INT_KERNEL,
	FLOAT_KERNEL
} CPI_KernelType;

typedef enum
{
	BLACK,
	MIRROR,
	BORDER
} CPI_Edging;

typedef enum
{
	ONOFF,
	MIX,
	NONE
} CPI_MaskType;

typedef enum
{
	ROBERTS,
	SOBEL,
	KIRSCH
} CPI_EdgeMethod;

typedef enum
{
	POINT_SAMPLE,
	BILINEAR,
	BEST_NATURAL,
	BEST_CGI,
	FILTERED
} CPI_FilterOp;

typedef enum
{
	FILTER_BOX,
	FILTER_TRIANGLE,
	FILTER_QUADRATIC,
	FILTER_CUBIC,
	FILTER_CATMULL,
	FILTER_MITCHELL,
	FILTER_GAUSSIAN,
	FILTER_SINC,
	FILTER_BESSEL,
	FILTER_LANCZOS2,
	FILTER_LANCZOS3
} CPI_FilterType;

typedef enum
{
	HANN,
	HAMMING,
	BLACKMAN,
	KAISER
} CPI_WindowType;

typedef enum
{
	SRT,
	RST,
	TSR,
	TRS,
	STR,
	RTS
} CPI_TransformOrder;

// Note: Please keep these alphabetical
/*****************************************/

	
typedef struct
{
	CPI_Bool	 premultiplyA;
	CPI_Bool	 premultiplyB;
} atopParms;


/*****************************************/


typedef struct
{
	CPI_Int32	 bitdepth;
	CPI_Float32	 blackIn[5];
	CPI_Float32	 blackOut[5];
	CPI_Float32	 whiteIn[5];
	CPI_Float32	 whiteOut[5];
} bitDepthParms;


/*****************************************/


typedef struct
{
	CPI_Float32	 	 kernelSize[2];
	CPI_KernelShape	 kernelShape;
	CPI_KernelType	 kernelType;
	CPI_Edging		 edgeType;
	CPI_Int32	 	 size[2];			/* Size of the area to blur */
	CPI_Int32	 	 offset[2];			/* Offset of the area to blur */
	CPI_Int32	 	 channelSelect;
	CPI_Int32	 	 maskChannel;
	CPI_MaskType	 maskType;
} blurParms;



/*****************************************/


typedef struct
{
	CPI_Float32	 	 brightness[5];
	CPI_Float32	 	 offset[5];
	CPI_Int32	 	 channelSelect;
	CPI_Int32	 	 maskChannel;
	CPI_MaskType	 maskType;
} brightnessParms;


/*****************************************/


typedef struct
{
	CPI_Int32	 outputChannels;
	CPI_Int32	 inputChannels[10];

	/* NOTE: inputChannels are specified as follows : */
	/* 		 0	= Black */
	/*		 1  = White */
	/*		 2  = Input 1 Channel 0 */
	/*		 3  = Input 1 Channel 1 */
	/*		 .. etc .. */
	/*		 12 = Input 1 Luminance */
	/*		 13 = Input 2 Channel 0 */
	/*		 14	= Input 2 Channel 1 */
	/*		 .. etc .. */
	/*		 23 = Input 2 Luminance */	 
} channelSwapParms;


/*****************************************/


typedef struct
{
	CPI_Float32		 softness;
	CPI_Float32		 hueMin;
	CPI_Float32		 hueMax;
	CPI_Float32		 saturationMin;
	CPI_Float32		 saturationMax;
	CPI_Float32		 luminanceMin;
	CPI_Float32		 luminanceMax;
} chromakeyParms;


/*****************************************/


typedef struct
{
	CPI_Int32	 size[2];
	CPI_Int32	 channels;
	CPI_Int32	 bitdepth;
	CPI_Float32	 color[5];
} colorParms;


/*****************************************/


typedef struct
{
	CPI_Float32	 	 contrast[5];
	CPI_Float32	 	 pivot[5];
	CPI_Int32	 	 channelSelect;
	CPI_Int32	 	 maskChannel;
	CPI_MaskType	 maskType;
} contrastParms;


/****************************************/


typedef struct
{
	CPI_Int32	 	 kernelSize[2];
	CPI_Float32		*kernel;
	CPI_Edging		 edgeType;
	CPI_Int32	 	 channelSelect;
	CPI_Int32	 	 maskChannel;
	CPI_MaskType 	 maskType;
} convolveParms;


/****************************************/


typedef struct
{
	CPI_Float32	 magnitude;
	CPI_Float32	 softness;
	CPI_Int32	 size[2];
	CPI_Int32	 offset[2];
	CPI_Int32	 channelSelect;
	CPI_Int32	 maskChannel;
	CPI_MaskType maskType;
} dilateParms;


/****************************************/


typedef struct
{
	CPI_Float32	 dissolve;
} dissolveParms;


/****************************************/


typedef struct
{
	CPI_Int32	 	 kernelSize;
	CPI_EdgeMethod	 edgeMethod;
	CPI_Int32	 	 size[2];
	CPI_Int32	 	 offset[2];
	CPI_Edging		 edgeType;
	CPI_Int32	 	 channelSelect;
	CPI_Int32	 	 maskChannel;
	CPI_MaskType	 maskType;
} edgeParms;


/****************************************/


typedef struct
{
	CPI_Float32	 	 magnitude;
	CPI_Float32	 	 softness;
	CPI_Int32	 	 size[2];
	CPI_Int32	 	 offset[2];
	CPI_Int32	 	 channelSelect;
	CPI_Int32	 	 maskChannel;
	CPI_MaskType	 maskType;
} erodeParms;


/****************************************/


typedef struct
{
	CPI_Int32	pivot;
} flipParms;


/****************************************/


typedef struct
{
	CPI_Float32	 gamma[5];
	CPI_Int32	 channelSelect;
	CPI_Int32	 maskChannel;
	CPI_MaskType maskType;
} gammaParms;


/****************************************/


typedef struct
{
	CPI_Float32	 hue;
	CPI_Float32	 saturation;
	CPI_Float32	 adjust;
	CPI_Bool	 isHSL;		/* Otherwise will use HSV space */
	CPI_Int32	 maskChannel;
	CPI_MaskType maskType;
} hueAdjustParms;


/****************************************/


typedef struct
{
	CPI_Bool	 premultiplyA;
} insideParms;


/****************************************/


typedef struct
{
	CPI_Int32	 channelSelect;
	CPI_Int32	 maskChannel;
	CPI_MaskType maskType;
} invertParms;


/****************************************/


typedef struct
{
	CPI_Int32		 maskChannel;
	CPI_MaskType	 maskType;
} maskParms;


/****************************************/


typedef struct
{
	CPI_Float32	 amount;
	CPI_Int32	 channelSelect;
	CPI_Int32	 maskChannel;
	CPI_MaskType maskType;
} monochromeParms;


/****************************************/


typedef struct
{
	CPI_Bool	 premultiplyA;
} outsideParms;


/****************************************/


typedef struct
{
	CPI_Bool	 premultiplyA;
	CPI_Bool	 premultiplyB;
} overParms;


/****************************************/


typedef struct
{
	CPI_Float32	 	 scale[2];
	CPI_Float32	 	 blurFactor;
	CPI_Float32	 	 mitchellB;
	CPI_Float32	 	 mitchellC;
	CPI_Float32	 	 kaiserA;
	CPI_FilterOp	 filterOp;
	CPI_FilterType	 filterType;
	CPI_WindowType	 windowType;
} resizeParms;


/****************************************/


typedef struct
{
	CPI_Float32	 	 rotateZ;
	CPI_Float32	 	 pivot[2];
	CPI_Float32	 	 blurFactor;
	CPI_Float32	 	 mitchellB;
	CPI_Float32	 	 mitchellC;
	CPI_Float32	 	 kaiserA;
	CPI_FilterOp	 filterOp;
	CPI_FilterType	 filterType;
	CPI_WindowType	 windowType;
} rotateParms;


/****************************************/


typedef struct
{
	CPI_Float32	 amount;
	CPI_Bool	 horizontal;	// if CPI_FALSE, vertical is assumed
} skewParms;


/****************************************/


typedef struct
{
	CPI_TransformOrder	 order;
	CPI_Float32		 	 translation[2];
	CPI_Float32	 	 	 pivot[2];
	CPI_Float32		 	 rotateZ;
	CPI_Float32	 	 	 scale[2];
	CPI_Float32		 	 blurFactor;
	CPI_Float32		 	 mitchellB;
	CPI_Float32		 	 mitchellC;
	CPI_Float32		 	 kaiserA;
	CPI_FilterOp	 	 filterOp;
	CPI_FilterType 	 	 filterType;
	CPI_WindowType 	 	 windowType;
} transformParms;


/****************************************/


typedef struct
{
	CPI_Float32	 	 translate[2];
	CPI_Float32	 	 blurFactor;
	CPI_Float32	 	 mitchellB;
	CPI_Float32	 	 mitchellC;
	CPI_Float32	 	 kaiserA;
	CPI_FilterOp	 filterOp;
	CPI_FilterType	 filterType;
	CPI_WindowType	 windowType;	
} translateParms;


/****************************************/


typedef struct
{
	CPI_Bool	 premultiplyA;
	CPI_Bool	 premultiplyB;
} underParms;


/****************************************/


typedef struct
{
	CPI_Float32	 	 scale[2];
	CPI_Float32	 	 pivot[2];
	CPI_Float32	 	 blurFactor;
	CPI_Float32	 	 mitchellB;
	CPI_Float32	 	 mitchellC;
	CPI_Float32	 	 kaiserA;
	CPI_FilterOp	 filterOp;
	CPI_FilterType	 filterType;
	CPI_WindowType	 windowType;
} zoomParms;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_InternalOps_h_ */
